@@ -1,7 +1,7 @@
 #-*- coding:utf-8 -*-
 
 #  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009-2011  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2012  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,24 +19,7 @@
 
 # Ported from GNUbik
 # Original filename: widget-set.c
-# Original copyright and license:
-#/*
-#    GNUbik -- A 3 dimensional magic cube game.
-#    Copyright (C) 2003, 2004  John Darrington
-#
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 3 of the License,  or
-#    (at your option) any later version.
-#
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License
-#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#*/
+# Original copyright and license: 2003, 2004  John Darrington, GPL3+
 
 
 import os
@@ -48,8 +31,8 @@ from gtk import gdk
 from .debug import *
 from . import config
 from . import dialogs
-from . import colour_sel
-from . import plugin_hooks
+from . import dialogcolors
+from . import plugins
 import state
 
 from . import glarea
@@ -64,7 +47,7 @@ class Application (object):
         
         #
         confstore.init_notify()
-        self.plugin_helper = plugin_hooks.PluginHelper(self)
+        self.plugin_helper = plugins.PluginHelper(self)
         
         # UI
         ui_file = os.path.join(config.UI_DIR, 'pybik.ui')
@@ -84,133 +67,68 @@ class Application (object):
         self.window = self.ui_xml.get_object('window_main')
         self.statusbar = self.ui_xml.get_object('statusbar_main')
         self.entry_formula = self.ui_xml.get_object('entry_formula')
+        self.treeview = self.ui_xml.get_object('treeview_scripts')
+        self.selected_tree_path = None
         
         self.window.set_icon_from_file(config.IMAGE_FILE)
-        
-        self.window.connect("delete-event", self.gtk_main_quit)
+        self.window.connect("delete-event", self.on_window_delete_event)
         
         self.create_menu()
         self.create_sidepane()
         
         self.cube_area = glarea.CubeArea()
         drawingarea_cube = self.ui_xml.get_object('drawingarea_cube')
-        self.cube_area.init(drawingarea_cube, self)
-        self.cube_area.connect('texture-changed', self.on_texture_changed)
+        self.cube_area.init(drawingarea_cube)
+        self.cube_area.connect('texture-changed', self.on_cubearea_texture_changed)
+        self.cube_area.connect('end-animation', self.on_cubearea_end_animation)
+        self.cube_area.connect('request-rotation', self.on_cubearea_request_rotation)
+        
+        if not os.path.exists(config.get_data_home()):
+            os.mkdir(config.get_data_home())
+        accel_map_file = os.path.join(config.get_data_home(), 'accels')
+        gtk.accel_map_load(accel_map_file)
+        gtk.accel_map_get().connect('changed', self.on_accel_map_changed, accel_map_file)
+        
+    def on_accel_map_changed(self, unused_map, unused_path, unused_key, unused_mods, filename):
+        gtk.accel_map_save(filename)
+        
+    def on_cubearea_texture_changed(self, unused_widget):
+        if dialogcolors.ColorSelectDialog.this_dlg:
+            dialogcolors.ColorSelectDialog.this_dlg.redraw_swatches()
+    def on_cubearea_end_animation(self, unused_cubearea, last_move):
+        self.cube_area.apply_to_glmodel(self.current_movement.current_cube_state)
+        self.cube_area.update_selection()
+        
+        if not last_move:
+            # go again
+            self.update_statusbar()
+            move_data = self.current_movement.request_play()
+            if not self.start_animation(move_data, stop_after=False):
+                self.update_ui()
+        else:
+            self.update_ui()
+    def on_cubearea_request_rotation(self, unused_cubearea, maxis, mslice, mdir):
+        self.current_movement.request_rotation(maxis, mslice, mdir)
+        self.update_statusbar()
+        move_data = self.current_movement.request_play()
+        self.start_animation(move_data, stop_after=False)
         
-    def on_texture_changed(self, unused_widget):
-        if colour_sel.ColorSelectDialog.this_dlg:
-            colour_sel.ColorSelectDialog.this_dlg.redraw_swatches()
-            
     def create_menu(self):
-        ui_def = ('''
-            <ui>
-                <menubar name="menubar1">
-                    <menu action="menu_game">
-                        <menuitem name="menu_new_random" action="action_new" />
-                        <menuitem name="menu_new_solved" action="action_new_solved" />
-                        <separator/>
-                        <menuitem name="menu_quit" action="action_quit" />
-                    </menu>
-                    <menu action="menu_edit">
-                        <menuitem name="menu_colors" action="action_colors" />
-                        <menuitem name="menu_preferences" action="action_preferences" />
-                    </menu>
-                    <menu action="menu_view">
-                        <menuitem name="menu_play_toolbar" action="toggleaction_play_toolbar" />
-                        <menuitem name="menu_statusbar" action="toggleaction_statusbar" />
-                    </menu>
-                    <menu action="menu_help">
-                        <menuitem name="menu_info" action="action_info" />
-                    </menu>
-                </menubar>
-                <toolbar name="toolbar_play">
-                    <toolitem name="toolbutton_rewind"  action="action_rewind" />
-                    <toolitem name="toolbutton_previous" action="action_previous" />
-                    <toolitem name="toolbutton_stop"    action="action_stop" />
-                    <toolitem name="toolbutton_play"    action="action_play" />
-                    <toolitem name="toolbutton_next"    action="action_next" />
-                    <toolitem name="toolbutton_forward" action="action_forward" />
-                    <separator />
-                    <toolitem name="toolbutton_add_mark" action="action_add_mark" />
-                    <toolitem name="toolbutton_remove_mark" action="action_remove_mark" />
-                </toolbar>
-                <accelerator action="action_rewind" />
-                <accelerator action="action_forward" />
-                <accelerator action="action_focus_formula" />
-            </ui>
-        ''')
+        ui_file = os.path.join(config.UI_DIR, 'menu.ui')
         
-        self.main_actions = gtk.ActionGroup('main_actions')
-        self.play_actions = gtk.ActionGroup('play_actions')
-        #name,           stock_id,  label,          accel,      tooltip,    callback
-        self.main_actions.add_actions([
-        ("menu_game",    None,      _('_Game')),
-        ("menu_edit",    None,      _('_Edit')),
-        ("menu_view",    None,      _('_View')),
-        ("menu_help",    None,      _('_Help')),
-        
-        ("action_new",  'gtk-new',  _("_New random"),  None,
-                                    _('New random cube'),   self.on_action_new_activate),
-        ("action_new_solved",None,  _("New _solved"),  '<Ctrl><Shift>n',
-                                    _('New solved cube'),   self.on_action_new_solved_activate),
-        ("action_quit", "gtk-quit", None,           None,       None,       self.gtk_main_quit),
-        
-        ("action_preferences","gtk-preferences",None,None,      None,
-                                                            self.on_action_preferences_activate),
-        ("action_colors","gtk-select-color",None,   None,       None,   self.on_action_colors_activate),
-        ("action_info", 'gtk-about',None,           None,       None,   self.on_action_info_activate),
-        
-        ('action_focus_formula', None, None,        '<Ctrl>l',  None,   self.on_action_focus_formula),
-        ])
-        
-        #name,          stock_id,   label,          accelerator,tooltip,   callback,   isactive
-        self.main_actions.add_toggle_actions([
-        ("toggleaction_play_toolbar",None,_("_Play Toolbar"),None,
-                                    _("Play Toolbar"), self.on_toggleaction_play_toolbar_toggled,   True),
-        ("toggleaction_statusbar",None,_("_Status Bar"),None,
-                                    _("Status Bar"),   self.on_toggleaction_statusbar_toggled,      True),
-        ])
-        
-        self.play_actions.add_actions([
-        #name,              stock_id,               label,      accel,      tooltip,    callback
-        ("action_rewind",   "gtk-media-rewind",     None,       '<Ctrl>r',
-                    _("Go to the previous mark (or the beginning) of the sequence of moves"),
-                    self.on_action_rewind_activate),
-        ("action_previous", "gtk-media-previous",   None,       None,
-                    _("Make one step backwards"),              self.on_action_previous_activate),
-        ("action_stop",     "gtk-media-stop",       None,       None,
-                    _("Stop running the sequence of moves"),   self.on_action_stop_activate),
-        ("action_play",     "gtk-media-play",       None,       None,
-                    _("Run forward through the sequence of moves"),    self.on_action_play_activate),
-        ("action_next",     "gtk-media-next",       None,       None,
-                    _("Make one step forwards"),               self.on_action_next_activate),
-        ("action_forward",  "gtk-media-forward",    None,       '<Ctrl><Shift>r',
-                    _("Go to the next mark (or the end) of the sequence of moves"),
-                    self.on_action_forward_activate),
-        ("action_add_mark", "gtk-add",              _("Add mark"),     None,
-                    _("Mark the current place in the sequence of moves"),
-                    self.on_action_add_mark_activate),
-        ("action_remove_mark",  "gtk-remove",       _('Remove mark'),   None,
-                    _("Remove the mark at the current place in the sequence of moves"),
-                    self.on_action_remove_mark_activate),
-        ])
+        self.main_actions = self.ui_xml.get_object('actiongroup_main')
+        self.play_actions = self.ui_xml.get_object('actiongroup_play')
         
         self.uimanager = gtk.UIManager()
         self.uimanager.insert_action_group(self.main_actions)
         self.uimanager.insert_action_group(self.play_actions)
-        self.uimanager.add_ui_from_string(ui_def)
+        self.uimanager.add_ui_from_file(ui_file)
         
         menubar = self.uimanager.get_widget('/menubar1')
         vbox = self.ui_xml.get_object('vbox1')
         vbox.pack_start(menubar, fill=False, expand=False)
         vbox.reorder_child(menubar, 0)
         
-        self.play_toolbar = self.uimanager.get_widget('/toolbar_play')
-        self.play_toolbar.get_accessible().set_name('ToolbarPlay')
-        vbox = self.ui_xml.get_object('vbox2')
-        vbox.pack_start(self.play_toolbar, fill=False, expand=False)
-        vbox.reorder_child(self.play_toolbar, 0)
-        
         accel = self.uimanager.get_accel_group()
         self.window.add_accel_group(accel)
     
@@ -219,40 +137,50 @@ class Application (object):
         
     @debug_func
     def create_sidepane(self):
-        # Get the menu structure asked for by the collection of plugins.
-        list_ = self.plugin_helper.startup_plugins()
-        
-        treeview = self.ui_xml.get_object('treeview_scripts')
-        treestore = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
-        treeview.set_model(treestore)
-        
         column = gtk.TreeViewColumn(_('Name'), gtk.CellRendererText(), text=0)
         column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
         column.set_fixed_width(200)
-        treeview.append_column(column)
+        self.treeview.append_column(column)
         
-        # add the items
-        tree_iters = {'':None}
-        for func_name, func in list_:
-            p = func_name.split('/', 4)
+        self.add_scripts_to_sidepane()
+        
+        self.treeview.connect('row-activated', self.on_treeview_row_activated)
+        self.id_treeview_motion_notify_event = None
+        active = confstore.dynamic_script_selection
+        self.ui_xml.get_object('toggleaction_dynamic_script_selection').set_active(active)
+        #self.treeview.expand_all()
+        self.treeview.show_all()
+        
+    def add_scripts_to_sidepane(self):
+        treestore = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
+        plugins = self.plugin_helper.load_plugins()
+        tree_iters = {():None}
+        for plugin_path, func in plugins:
             for i in xrange(1, 5):
-                p_i = p[:i]
-                p_is = '/'.join(p_i)
-                p_i_p = p[:i-1]
-                p_is_p = '/'.join(p_i_p)
-                name = p_i[-1]
-                if p_is not in tree_iters:
-                    #debug('|'.join((name, p_is, func_name)))
-                    ti = treestore.append(tree_iters[p_is_p], [name, func if p_is==func_name else None])
-                    tree_iters[p_is] = ti
-        treeview.connect('row-activated', self.on_run_script)
-        #treeview.expand_all()
-        treeview.show_all()
-
+                p_i = plugin_path[:i]
+                if p_i not in tree_iters:
+                    name = _(p_i[-1])
+                    tree_iters[p_i] = treestore.append(
+                                        tree_iters[plugin_path[:i-1]],
+                                        [name, func if p_i == plugin_path else None])
+        self.treeview.set_model(treestore)
+        
     def is_animating(self):
         return self.cube_area.animation_in_progress
         
-    def on_run_script(self, treeview, path, unused_view_column):
+    def on_treeview_motion_notify_event(self, treeview, event):
+        tpath_column = treeview.get_path_at_pos(int(event.x), int(event.y))
+        if tpath_column is None or self.selected_tree_path == tpath_column[0]:
+            return True
+        tpath = tpath_column[0]
+        treeview.collapse_all()
+        treeview.expand_to_path(tpath)
+        treeview.expand_row(tpath, False)
+        treeview.set_cursor(tpath)
+        self.selected_tree_path = tpath
+        return True
+        
+    def on_treeview_row_activated(self, treeview, path, unused_view_column):
         if self.is_animating():
             return
         unused_name, func = treeview.get_model()[path]
@@ -260,13 +188,11 @@ class Application (object):
             self.cube_area.apply_to_glmodel(self.current_movement.current_cube_state)
             self.update_ui()
             move_data = self.current_movement.request_play()
-            if move_data:
-                self.cube_area.animate_rotation(self, move_data)
-            else:
+            if not self.start_animation(move_data, stop_after=False):
                 self.cube_area.render()
                 
     @debug_func
-    def start_main_loop(self):
+    def run_main_loop(self):
         self.window.show()
         gtk.quit_add(0, self.on_quit)
         gtk.main()
@@ -322,78 +248,105 @@ class Application (object):
     # If data is non zero,  then all it's data will be reallocated
     @debug_func
     def request_new_game(self, unused_widget=None, new_size=None, solved=None):
-        self.current_movement.request_stop()
+        #FIXME: fix abort_requested and use it here
         if self.is_animating():
-            self.current_movement.request_abort()
+            return
         gobject.idle_add(self.new_game, new_size, solved)
-    
-    
+        
+    def start_animation(self, move_data, stop_after):
+        if move_data:
+            blocks = list(self.current_movement.current_cube_state.identify_rotation_blocks(
+                                move_data.axis, move_data.slice))
+            self.cube_area.animate_rotation(move_data, blocks, stop_after)
+            self.playbarstate.playing(self.current_movement.mark_before)
+            return True
+        return False
+        
+        
     # Action handlers
     @debug_func
     def on_quit(self):
         (unused_dimension, confstore.saved_state, confstore.saved_moves,
                     confstore.saved_pos) = self.current_movement.get_saved_game()
         return 0
-    gtk_main_quit = gtk.main_quit
+    on_window_delete_event = gtk.main_quit
     def on_action_new_activate(self, *unused_args):
         self.request_new_game(solved=False)
     def on_action_new_solved_activate(self, *unused_args):
         self.request_new_game(solved=True)
     def on_action_colors_activate(self, unused_widget):
-        colour_sel.open_ColorSelectDialog(self)
+        dialogcolors.colorselect_dialog(self)
     def on_action_preferences_activate(self, unused_widget):
-        dialogs.preferencesdialog(self)
+        dialogs.preferences_dialog(self)
     def on_toggleaction_play_toolbar_toggled(self, widget):
         self.play_toolbar.props.visible = widget.props.active
     def on_toggleaction_statusbar_toggled(self, widget):
         self.statusbar.props.visible = widget.props.active
+    def on_action_reset_rotation_activate(self, unused_widget):
+        self.cube_area.reset_rotation()
+    def on_toggleaction_dynamic_script_selection_toggled(self, widget):
+        confstore.dynamic_script_selection = widget.props.active
+        if widget.props.active:
+            self.id_treeview_motion_notify_event = (
+                self.treeview.connect('motion-notify-event', self.on_treeview_motion_notify_event))
+        else:
+            self.treeview.disconnect(self.id_treeview_motion_notify_event)
+            self.id_treeview_motion_notify_event = None
     def on_action_info_activate(self, unused_widget):
         dialogs.show_about(self.window)
         
-    def on_action_focus_formula(self, *unused_args):
-        self.entry_formula.grab_focus()
-        
     def on_action_rewind_activate(self, unused_widget):
         if not self.is_animating():
             needs_update = self.current_movement.do_rewind_to_mark()
             self.cube_area.apply_to_glmodel(self.current_movement.current_cube_state)
             if needs_update:
-                #TODO: render updates internal cube state in at idle time
-                # maybe that is the reason for the following bug
-                # internal state should applied before gobject.idle_add
-                # this is also for do_fast_forward
                 self.cube_area.render()
-                #FIXME: sometimes 'rotated center' is shown by mistake instead of 'solved'
-                gobject.idle_add(self.update_ui)
+                self.update_ui()
     def on_action_previous_activate(self, unused_widget):
         if not self.is_animating():
             move_data = self.current_movement.request_back()
-            if move_data:
-                self.cube_area.animate_rotation(self, move_data)
+            self.start_animation(move_data, stop_after=True)
     def on_action_stop_activate(self, unused_widget):
-        self.current_movement.request_stop()
+        self.cube_area.stop_requested = 1
     def on_action_play_activate(self, unused_widget):
         if not self.is_animating():
             move_data = self.current_movement.request_play()
-            if move_data:
-                self.cube_area.animate_rotation(self, move_data)
+            self.start_animation(move_data, stop_after=False)
     def on_action_next_activate(self, unused_widget):
         if not self.is_animating():
             move_data = self.current_movement.request_next()
-            if move_data:
-                self.cube_area.animate_rotation(self, move_data)
+            self.start_animation(move_data, stop_after=True)
     def on_action_forward_activate(self, unused_widget):
+        if self.is_animating():
+            return
         self.current_movement.do_fast_forward()
         self.cube_area.apply_to_glmodel(self.current_movement.current_cube_state)
         self.cube_area.render()
-        gobject.idle_add(self.update_ui)
+        self.update_ui()
     def on_action_add_mark_activate(self, unused_widget):
         self.current_movement.request_mark_move_queue(True)
         self.update_ui()
     def on_action_remove_mark_activate(self, unused_widget):
         self.current_movement.request_mark_move_queue(False)
         self.update_ui()
-    
+        
+    def on_action_set_as_initial_state_activate(self, unused_widget):
+        if self.is_animating():
+            return
+        self.current_movement.set_as_initial_state()
+        self.update_ui()
+    def on_action_reload_scripts_activate(self, unused_widget):
+        self.plugin_helper = plugins.PluginHelper(self)
+        self.add_scripts_to_sidepane()
+    def on_action_invert_moves_activate(self, unused_widget):
+        if self.is_animating():
+            return
+        self.current_movement.invert_moves()
+        self.cube_area.apply_to_glmodel(self.current_movement.current_cube_state)
+        self.cube_area.render()
+        self.update_ui()
+        
+        
     #Callbacks for PreferencesDialog
     def on_button_size_reset_activate(self, unused_widget):
         del confstore.dimension
@@ -401,26 +354,26 @@ class Application (object):
         del confstore.frameQty
     def on_button_lighting_reset_activate(self, unused_widget):
         del confstore.lighting
-    
-    @debug_func
-    def on_colour_select(self, unused_widget):
-        colour_sel.open_ColorSelectDialog(self)
+        
     def on_button_color_reset_activate(self, unused_widget):
-        i = colour_sel.ColorSelectDialog.this_dlg.active_face[0]
+        i = dialogcolors.ColorSelectDialog.this_dlg.active_face[0]
         del confstore.colors[i].color
     def on_button_pattern_reset_activate(self, unused_widget):
-        i = colour_sel.ColorSelectDialog.this_dlg.active_face[0]
+        i = dialogcolors.ColorSelectDialog.this_dlg.active_face[0]
         del confstore.colors[i].pattern
     def on_button_image_reset_activate(self, unused_widget):
-        i = colour_sel.ColorSelectDialog.this_dlg.active_face[0]
+        i = dialogcolors.ColorSelectDialog.this_dlg.active_face[0]
         del confstore.colors[i].facetype
         del confstore.colors[i].imagefile
         del confstore.colors[i].imagemode
-
-    
+    def on_button_background_color_set(self, button):
+        confstore.background_color = str(button.get_color())
+    def on_button_background_reset_activate(self, unused_widget):
+        del confstore.background_color
+        
+        
     # Callbacks for editing formulas
     def on_entry_formula_activate(self, unused_widget=None):
-        # while animating leads to a invalid cube
         if self.is_animating():
             #TODO: Better: stop animating and proceed
             return
@@ -429,24 +382,20 @@ class Application (object):
         self.current_movement.set_from_formula(code, pos)
         self.cube_area.apply_to_glmodel(self.current_movement.current_cube_state)
         self.cube_area.render()
-        gobject.idle_add(self.update_ui)
+        self.update_ui()
         
     def on_entry_formula_icon_press(self, unused_widget, icon_info, event):
-        # while animating leads to a invalid cube
         if self.is_animating():
             #TODO: Better: stop animating and proceed
             return
         if event.button==1 and event.type==gdk.BUTTON_PRESS:
             if icon_info.value_nick == 'primary':
-                #debug('activate')
                 return self.on_entry_formula_activate()
             if icon_info.value_nick == 'secondary':
-                #debug('clear')
                 self.entry_formula.set_text('')
                 self.on_entry_formula_activate()
         
-    def set_entry_formula(self, code, pos, unused_markup):
-        #debug('markup',markup)
+    def set_entry_formula(self, code, pos, unused_markup=None):
         self.entry_formula.set_text(code)
         self.entry_formula.set_position(pos)
         #TODO: seems, that gtk.Entry doesn't work with pango markup, use one-line gtk.TextView?
@@ -495,7 +444,8 @@ class Application (object):
                 self.playbarstate.mid(self.current_movement.all_moves.prev().mark_after)
                 
         # update formula
-        code, pos, markup = self.current_movement.all_moves.format(self.current_movement.current_cube_state.dimension)
+        code, pos, markup = self.current_movement.all_moves.format(
+                                    self.current_movement.current_cube_state.dimension)
         self.set_entry_formula(code, pos, markup)
         
         self.update_statusbar(first=first)
