@@ -1,7 +1,7 @@
 #-*- coding:utf-8 -*-
 
 #  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009-2011  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2012  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,24 +19,7 @@
 
 # Ported from GNUbik
 # Original filename: cube.c
-# Original copyright and license:
-#/*
-#  Copyright (c) 2004        Dale Mellor,  John Darrington
-#  copyright (c) 1998,  2003  John Darrington
-#
-#  This program is free software; you can redistribute it and/or modify
-#  it under the terms of the GNU General Public License as published by
-#  the Free Software Foundation; either version 3 of the License,  or
-#  (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#*/
+# Original copyright and license:  1998, 2003, 2004  Dale Mellor, John Darrington,  GPL3+
 
 
 # This line makes pyrex happy
@@ -411,11 +394,13 @@ def init_turn_axes_4(v, w1, w2, w3, w4):
     init_vector(v[3], w4)
 
 #pxd ctypedef Vector turn_axes_t[6][4]
+#pxd ctypedef Vector turn_axes_center_t[6]
 #px-
-turn_axes_t = None
-cython.declare(turn_axes = turn_axes_t)
-#px-
+turn_axes_t = turn_axes_center_t = None
+cython.declare(turn_axes = turn_axes_t, turn_axes_center = turn_axes_center_t)
+#px-2
 turn_axes = [Matrix() for __i in range(6)]
+turn_axes_center = [Vector() for __i in range(6)]
 
 turn_axesX  = [ 1, 0, 0, 0]
 turn_axes_X = [-1, 0, 0, 0]
@@ -431,6 +416,13 @@ init_turn_axes_4(turn_axes[3], turn_axes_Z, turn_axesX, turn_axesZ, turn_axes_X)
 init_turn_axes_4(turn_axes[4], turn_axes_Y, turn_axes_Z, turn_axesY, turn_axesZ)
 init_turn_axes_4(turn_axes[5], turn_axesY, turn_axes_Z, turn_axes_Y, turn_axesZ)
 
+init_vector(turn_axes_center[0], turn_axesZ)
+init_vector(turn_axes_center[1], turn_axes_Z)
+init_vector(turn_axes_center[2], turn_axesY)
+init_vector(turn_axes_center[3], turn_axes_Y)
+init_vector(turn_axes_center[4], turn_axesX)
+init_vector(turn_axes_center[5], turn_axes_X)
+
 # Convert a vector to a axis number.  Obviously this assumes the vector
 #is orthogonal to the frame of reference ( another nasty kludge ).
 #pxd cdef int get_turn_axis(float* vector)
@@ -475,6 +467,25 @@ def get_selected_move(block, face, quadrant):
     dir_ = get_turn_dir(vector)
     return axis, slice_, dir_
     
+def get_selected_move_center(block, face):
+    cython.declare(
+        vector = Vector,
+        pv = cython.p_float,
+        t = p_Vector)
+    #px-
+    vector = Vector()
+    
+    pv = turn_axes_center[face]
+    # Fetch the selected block's transformation from its original orientation
+    t = _get_block_transform(block)
+    # transform it, so that we go the right way
+    transform(vector, t, pv)
+    
+    axis = get_turn_axis(vector)
+    slice_ = get_turn_slice(block, axis)
+    dir_ = get_turn_dir(vector)
+    return axis, slice_, dir_
+    
 #pxd cdef inline p_Vector glGetProjectionMatrix(p_Vector m):
 #pxd    glGetFloatv(GL_PROJECTION_MATRIX, <float*>m)
 #pxd    return m
