@@ -27,7 +27,7 @@ def error(*args, **kwargs):
     
 debug = lambda *args, **kwargs: None
 debug_func = lambda x: x
-DEBUG_LEVEL = -1
+DEBUG_LEVEL = 0
 DEBUG_FUNC = DEBUG_MSG = DEBUG_ROTATE = DEBUG_DRAW = DEBUG_KEYS = DEBUG_TEST = DEBUG_PICK = False
 
 def install(module, debug_modes):
