@@ -19,24 +19,7 @@
 
 # Ported from GNUbik
 # Original filename: drwBlock.c
-# Original copyright and license:
-#/*
-#    The routines which actually draw the blocks of the cube.
-#    Copyright (C) 1998,  2003 John Darrington
-#
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 3 of the License,  or
-#    (at your option) any later version.
-#
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License
-#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#*/
+# Original copyright and license: 1998, 2003 John Darrington, GPL3+
 
 
 # This line makes pyrex happy
@@ -73,17 +56,13 @@ glMultMatrixM = glMultMatrixf
 
 
 # We use a little bit of glut in debug mode 
-#px-
-from OpenGL import GLUT as glut
+if DEBUG_DRAW:
+    from OpenGL import GLUT as glut
 def glutInit(argv):
-    #px-
-    glut.glutInit(len(argv), *argv[:])
-    pass
+    if DEBUG_DRAW:
+        glut.glutInit(len(argv), *argv[:])
 def renderString(string):
-    for c in string:
-        #px-
-        glut.glutStrokeCharacter(glut.GLUT_STROKE_MONO_ROMAN, ord(c))
-        pass
+    glut.glutStrokeString(glut.GLUT_STROKE_MONO_ROMAN, string)
 
 
 # this macro produces +1 if i is even. -1 if i is odd 
@@ -129,7 +108,6 @@ def set_quadrant_vector_vj(block, face, quadrant, vj, j):
 
 #pxd cdef void transform_vj(float *r, Matrix M, float vj, int j)
 def transform_vj(r, M, vj, j):
-    #assert len(r) == MATRIX_DIM
     i = cython.declare(cython.int)
     for i in range(MATRIX_DIM):
         r[i] = M[j][i] * vj
@@ -331,8 +309,8 @@ def draw_block(highlight, block_id, render_mode):
                 draw_label(i, block_id)
             else:
                 pick_face(i, block_id)
-            #px-
-            if render_mode and DEBUG_DRAW:  draw_face_debug(i, block_id)
+            if render_mode and DEBUG_DRAW:
+                draw_face_debug(i, block_id)
         elif anim_state.running:
             if render_mode:
                 draw_face(i, highlight, block_id)
@@ -612,22 +590,22 @@ def draw_cube_debug():
 #pxd cdef void draw_face_debug(int face, int block_id)
 def draw_face_debug(face, block_id):
     # render the block number 
-    glPushMatrix ()
-    glColor3f (0, 0, 0)
+    glPushMatrix()
+    glColor3f(0, 0, 0)
     
-    glTranslatef (-1,  -0.8, 0.1)
-    glScalef (0.01, 0.01, 0.01)
+    glTranslatef(-1,  -0.8, 0.02)
+    glScalef(0.0075, 0.0075, 0.0075)
     
     renderString("%d" % block_id)
-    glPopMatrix ()
+    glPopMatrix()
     
     # render the face number,  a little bit smaller
     # so we can see what's what. 
-    glPushMatrix ()
-    glTranslatef (+0.5,  +0.4, 0.1)
-    glScalef (0.005, 0.005, 0.005)
-    renderString("%d" % face)
-    glPopMatrix ()
+    glPushMatrix()
+    glTranslatef(-0.5, 0.0, 0.02)
+    glScalef(0.0075, 0.0075, 0.0075)
+    renderString(['U0','D1','L2','R3','F4','B5'][face])
+    glPopMatrix()
 
 
 #pxd cdef enum:
