@@ -1,7 +1,7 @@
 #-*- coding:utf-8 -*-
 
 #  Pybik -- A 3 dimensional magic cube game.
-#  Copyright © 2009-2011  B. Clausius <barcc@gmx.de>
+#  Copyright © 2009-2012  B. Clausius <barcc@gmx.de>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,29 +19,13 @@
 
 # Ported from GNUbik
 # Original filename: glarea.c
-# Original copyright and license:
-#/*
-#  GNUbik -- A 3 dimensional magic cube game.
-#  Copyright (C) 2003, 2004  John Darrington
-#
-#  This program is free software; you can redistribute it and/or modify
-#  it under the terms of the GNU General Public License as published by
-#  the Free Software Foundation; either version 3 of the License,  or
-#  (at your option) any later version.
-#
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#  GNU General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#*/
+# Original copyright and license: 2003, 2004  John Darrington,  GPL3+
 
+import os
+from collections import namedtuple
 
 import OpenGL.error
 from OpenGL import GLX as glx
-
 import gobject
 import gtk
 from gtk import gdk
@@ -50,23 +34,24 @@ from gtk import gdkgl
 import gio
 
 from .debug import *
+from . import config
 import cube
 import drwBlock
 import glarea_common
-import selection
-from . import cursors
 from . import textures
-from .confstore import confstore, COLORED, IMAGED
+from .confstore import confstore, COLORED, IMAGED, SELECTIONMODE_QUAD, SELECTIONMODE_EXT
+
 
 RDRAG_COLOUR = 0
 RDRAG_FILELIST = 1
 # picture rate in ms. 40ms = 25 frames per sec
 picture_rate = 40
 
-
 class CubeArea (gobject.GObject):
     __gsignals__ = {
         'texture-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ()),
+        'end-animation': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (bool,)),
+        'request-rotation': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (int,int,int)),
     }
     
     def __init__(self):
@@ -75,27 +60,35 @@ class CubeArea (gobject.GObject):
         self.widget = None
         self.last_mouse_x = -1
         self.last_mouse_y = -1
-        self.button_down_cube = False
+        self.default_rotation_x = 10.
+        self.default_rotation_y = 10.
+        self.rotation_x, self.rotation_y = glarea_common.set_rotation_xy(
+                                            self.default_rotation_x, self.default_rotation_y)
         self.button_down_background = False
-        self.renderPending = False
-        self.setcursorPending = False
+        self.render_pending = False
+        self.setcursor_pending = False
         self.animation_in_progress = 0
         self.timer = None
         # the time for which the mouse must stay still, for anything to happen.
         self.idle_threshold = 10
         self.mouse_xy = -1, -1
-        self.cursor_angle = 0.0
         # Structure to hold copy of the last selection taken or None
         self.current_selection = None
+        self.selection_mode = None
         self.enable_disable_selection_entered = 0
         self.stop_detected = False
         self.motion = False
+        self.stop_requested = False
+        self.abort_requested = False
         confstore.callbacks.append(self.on_confstore_changed)
         
+        self.cursors = None
+        self.load_cursors()
+        
     def is_item_selected(self):
         return self.current_selection is not None
         
-    def init(self, widget, app):
+    def init(self, widget):
         self.widget = widget
         
         # this is the number of frames drawn when a slice of the cube rotates 90 degrees
@@ -144,17 +137,13 @@ class CubeArea (gobject.GObject):
                                | gdk.POINTER_MOTION_MASK)
                                
         self.widget.connect("key-press-event", self.on_key_press_event)
-        if DEBUG_KEYS:
-            import testutils
-            self.widget.connect("key-press-event", testutils.debug_keys, app)
             
         self.widget.connect("motion-notify-event", self.on_motion_notify_event)
-        self.widget.connect("scroll-event", self.on_scroll_event)
-        self.widget.connect("button-press-event", self.on_button_event)
-        self.widget.connect("button-release-event", self.on_button_event)
-        self.widget.connect("button-press-event", self.on_button_press_event, app)
+        self.widget.connect("button-press-event", self.on_button_press_event)
+        self.widget.connect("button-release-event", self.on_button_release_event)
         
         # initialise the selection mechanism
+        self.set_selection_mode(confstore.selection_mode)
         self.timer = gobject.timeout_add(self.idle_threshold, self.on_timer_motion)
         # Add a handler to for all those occasions when we don't need the
         # selection mechanism going 
@@ -163,6 +152,26 @@ class CubeArea (gobject.GObject):
         self.widget.connect("visibility-notify-event", self.on_enter_leave_notify_event)
         self.widget.connect("unmap-event", self.on_enter_leave_notify_event)
         
+    def load_cursors(self):
+        display = gdk.display_get_default()
+        cursors = []
+        # Load 3 cursors from file (n - ne)
+        for i, (x, y) in enumerate([(8, 0), (11, 0), (15, 0)]):
+            filename = os.path.join(config.UI_DIR, 'mouse_{}.png'.format(i))
+            pixbuf = gdk.pixbuf_new_from_file(filename)
+            cursors.append((pixbuf, x, y))
+        # 1 cursor (nnw)
+        pixbuf, x, y = cursors[1]
+        cursors.insert(0, (pixbuf.flip(True), 15-x, y))
+        # 12 cursors (ene - nw)
+        for i in range(4,16):
+            pixbuf, x, y = cursors[-4]
+            cursors.append((pixbuf.rotate_simple(gdk.PIXBUF_ROTATE_CLOCKWISE), 15-y, x))
+        cursors.append(cursors[0])
+        self.cursors = [gdk.Cursor(display, pixbuf, x, y) for pixbuf, x, y in cursors[1:]]
+        self.cursors.append(gdk.Cursor(gdk.DOT))
+        self.cursors.append(gdk.Cursor(gdk.CROSSHAIR))
+        
     @staticmethod
     def set_the_colours():
         for i in xrange(6):
@@ -204,6 +213,8 @@ class CubeArea (gobject.GObject):
             self.__texInit_done = True
         glarea_common.graphics_area_init(confstore.lighting)
         self.set_the_colours()
+        color = gdk.Color(confstore.background_color)
+        glarea_common.set_background_color(color.red_float, color.green_float, color.blue_float)
         gldrawable.gl_end()
         
     @staticmethod
@@ -226,9 +237,6 @@ class CubeArea (gobject.GObject):
         glcontext = gtkgl.widget_get_gl_context(widget)
         gldrawable = gtkgl.widget_get_gl_drawable(widget)
         
-        #TODO ???
-        #if not gdkgl.Drawable.gl_begin(gldrawable, glcontext):
-        #    return False
         if not gdkgl.Drawable.make_current(gldrawable, glcontext):
             debug("Cannot set gl drawable current")
             return
@@ -243,44 +251,101 @@ class CubeArea (gobject.GObject):
         return False
         
     def on_key_press_event(self, unused_widget, event):
-        shifted = 0
-        if event.state & gdk.SHIFT_MASK:
-            shifted = 1
-        return self.do_keysyms(event.keyval, shifted)
-        
-    def do_keysyms(self, keysym, shifted):
-        if keysym == gtk.keysyms.Right:
-            if shifted:
-                dir_ = 0
-                axis = 2
-            else:
-                dir_ = 1
-                axis = 1
-        elif keysym == gtk.keysyms.Left:
-            if shifted:
-                dir_ = 1
-                axis = 2
-            else:
-                dir_ = 0
-                axis = 1
-        elif keysym == gtk.keysyms.Up:
-            axis = 0
-            dir_ = 0
-        elif keysym == gtk.keysyms.Down:
-            axis = 0
-            dir_ = 1
+        if event.keyval == gtk.keysyms.Right:
+            self.rotation_x += 1
+        elif event.keyval == gtk.keysyms.Left:
+            self.rotation_x -= 1
+        elif event.keyval == gtk.keysyms.Up:
+            self.rotation_y -= 1
+        elif event.keyval == gtk.keysyms.Down:
+            self.rotation_y += 1
         else:
-            return True
+            return False
             
-        self.rotate_cube(axis, dir_)
+        self.rotation_x, self.rotation_y = glarea_common.set_rotation_xy(
+                                                    self.rotation_x, self.rotation_y)
+        self.render()
         return True
         
+    FacetSelection = namedtuple('FacetSelection', 'block face quadrant maxis mslice mdir')
+    selectionmode_ext_map = {
+                    ( 0,0): 0,  ( 0,2): 3,  ( 0,4): 1,
+                    ( 1,0): 1,  ( 1,2): 3,
+                    ( 2,0): 1,  ( 2,2): 2,  ( 2,5): 2,
+                    ( 3,0): 0,              ( 3,4): 2,
+                    ( 4,0): 0,
+                    ( 5,0): 2,              ( 5,5): 2,
+                    ( 6,0): 3,  ( 6,3): 0,  ( 6,4): 2,
+                    ( 7,0): 3,  ( 7,3): 1,
+                    ( 8,0): 2,  ( 8,3): 1,  ( 8,5): 1,
+                                ( 9,2): 0,  ( 9,4): 1,
+                                (10,2): 0,
+                                (11,2): 2,  (11,5): 3,
+                                            (12,4): 0,
+                                            (14,5): 0,
+                                (15,3): 0,  (15,4): 3,
+                                (16,3): 0,
+                                (17,3): 2,  (17,5): 1,
+                    (18,1): 3,  (18,2): 0,  (18,4): 0,
+                    (19,1): 3,  (19,2): 1,
+                    (20,1): 2,  (20,2): 1,  (20,5): 3,
+                    (21,1): 0,              (21,4): 0,
+                    (22,1): 0,
+                    (23,1): 2,              (23,5): 0,
+                    (24,1): 0,  (24,3): 3,  (24,4): 3,
+                    (25,1): 1,  (25,3): 3,
+                    (26,1): 1,  (26,3): 2,  (26,5): 0,
+                }
+    def pick_polygons(self, x, y):
+        '''Identify the block at screen co-ordinates x,y.'''
+        
+        glcontext = gtkgl.widget_get_gl_context(self.widget)
+        gldrawable = gtkgl.widget_get_gl_drawable(self.widget)
+        
+        if not gdkgl.Drawable.make_current(gldrawable, glcontext):
+            debug("Cannot set gl drawable current")
+            return
+        
+        height = self.widget.allocation.height
+        
+        #TODO Fix pick mode 2 and remove pick mode 1
+        # CubeArea.update_selection uses (axis, slice, dir)
+        # and dialogcolors.drag_data_received uses face, so pick_polygons should return 4 values
+        closest = glarea_common.pick_polygons(confstore.lighting, x, height-y, 1)
+        #closest2 = glarea_common.pick_polygons(x, height-y, 1, 2)
+        if closest is None:
+            return None
+            
+        block = closest[0]
+        face = closest[1]
+        quadrant = closest[2]
+        
+        size = cube.get_cube_dimension()
+        if self.selection_mode == SELECTIONMODE_EXT and size > 1:
+            blockidx = [block % size, block/size % size, block/size/size % size]
+            blockidx = [0 if b==0 else 1 if b<size-1 else 2 for b in blockidx]
+            block3 = blockidx[0] + blockidx[1]*3 + blockidx[2]*9
+            if set(blockidx) <= {0,2}:
+                def fold(q, i):
+                    q = (q - i) % 4 * 2 - 3
+                    q = -abs(q)
+                    q = ((q + 3) // 2 + i) % 4
+                    return q
+                quadrant = fold(quadrant, self.selectionmode_ext_map[(block3, face)])
+                maxis, mslice, mdir = cube.get_selected_move(block, face, quadrant)
+            elif blockidx.count(1) == 2:
+                quadrant = None
+                maxis, mslice, mdir = cube.get_selected_move_center(block, face)
+            else:
+                quadrant = self.selectionmode_ext_map[(block3, face)]
+                maxis, mslice, mdir = cube.get_selected_move(block, face, quadrant)
+        else:
+            maxis, mslice, mdir = cube.get_selected_move(block, face, quadrant)
+        return self.FacetSelection(block, face, quadrant, maxis, mslice, mdir)
+        
     def update_selection(self):
         '''This func determines which block the mouse is pointing at'''
-        self.current_selection = selection.pick_polygons(self.widget, confstore.lighting, *self.mouse_xy)
-        if self.is_item_selected():
-            cs = self.current_selection
-            self.cursor_angle = cube.get_cursor_angle(cs.block, cs.face, cs.quadrant)
+        self.current_selection = self.pick_polygons(*self.mouse_xy)
         self.set_cursor()
 
     def on_motion_notify_event(self, unused_widget, event):
@@ -291,20 +356,15 @@ class CubeArea (gobject.GObject):
         if not self.button_down_background:
             return False
             
-        xmotion = 0
-        ymotion = 0
-        if self.last_mouse_x >= 0:
-            xmotion = event.x - self.last_mouse_x
-        if self.last_mouse_y >= 0:
-            ymotion = event.y - self.last_mouse_y
-            
-        self.last_mouse_x = event.x
-        self.last_mouse_y = event.y
-        
-        if ymotion > 0: self.rotate_cube(0, 1)
-        if ymotion < 0: self.rotate_cube(0, 0)
-        if xmotion > 0: self.rotate_cube(1, 1)
-        if xmotion < 0: self.rotate_cube(1, 0)
+        # perform rotation
+        offset_x = int(event.x) - self.last_mouse_x
+        offset_y = int(event.y) - self.last_mouse_y
+        self.rotation_x, self.rotation_y = glarea_common.set_rotation_xy(
+                                                        self.rotation_x + offset_x,
+                                                        self.rotation_y + offset_y)
+        self.rotation_x -= offset_x
+        self.rotation_y -= offset_y
+        self.render()
         return False
         
     def on_enter_leave_notify_event(self, unused_widget, event):
@@ -339,71 +399,62 @@ class CubeArea (gobject.GObject):
                 self.update_selection()
         self.motion = False
         return True
-
-    def on_scroll_event(self, unused_widget, event):
-        '''Rotate the cube about the z axis (relative to the viewer)'''
-        self.rotate_cube(2, not event.direction)
-        return False
         
-    def on_button_event(self, unused_widget, event):
-        # buttons 4 and 5 mean mouse wheel scrolling
-        if event.button == 4:
-            self.rotate_cube(2, 1)
-        if event.button == 5:
-            self.rotate_cube(2, 0)
-        if event.button != 1:
+    def on_button_press_event(self, widget, event):
+        '''handle mouse clicks'''
+        widget.grab_focus()
+        
+        if self.animation_in_progress:
             return False
             
-        if event.type == gdk.BUTTON_PRESS:
-            if self.is_item_selected():
-                self.button_down_cube = True
+        if self.is_item_selected():
+            # make a move
+            cs = self.current_selection
+            mslice = -1 if event.state & gdk.CONTROL_MASK else cs.mslice
+            if cs.quadrant is None and confstore.mouse_left_handed:
+                mdir = not cs.mdir
             else:
-                self.button_down_background = True
+                mdir = cs.mdir
+            if event.button == 1:
+                self.emit('request-rotation', cs.maxis, mslice, mdir)
+            elif event.button == 3 and confstore.selection_mode == SELECTIONMODE_EXT:
+                self.emit('request-rotation', cs.maxis, mslice, not mdir)
+        elif event.button == 1:
             # disable selection
             if self.timer is not None:
                 gobject.source_remove(self.timer)
                 self.timer = None
-        elif event.type == gdk.BUTTON_RELEASE:
-            # enable selection
-            if self.timer is None:
-                self.timer = gobject.timeout_add(self.idle_threshold, self.on_timer_motion)
-            self.button_down_cube = False
-            self.button_down_background = False
-        return False
-        
-    def on_button_press_event(self, widget, event, app):
-        if event.type == gdk.BUTTON_PRESS:
-            widget.grab_focus()
-            self.do_mouse_event(event.button, app)
+            # start rotation
+            self.button_down_background = True
+            self.last_mouse_x = int(event.x)
+            self.last_mouse_y = int(event.y)
+            
+        self.render()
         return True
         
-    def do_mouse_event(self, button, application):
-        '''handle mouse clicks'''
-        # Don't let a user make a move,  whilst one is already in progress,
-        # otherwise the cube falls to bits.
-        if self.animation_in_progress:
-            return
+    def on_button_release_event(self, unused_widget, event):
+        if event.button != 1:
+            return False
             
-        if button == 1:
-            # Make a move
-            if self.is_item_selected():
-                cs = self.current_selection
-                application.current_movement.request_rotation(cs.axis_o, cs.slice_o, cs.dir_o)
-                application.update_statusbar()
-                move_data = application.current_movement.request_play()
-                if move_data:
-                    self.animate_rotation(application, move_data)
-            self.render()
+        if self.button_down_background:
+            # end rotation
+            self.rotation_x += int(event.x) - self.last_mouse_x
+            self.rotation_y += int(event.y) - self.last_mouse_y
+            self.button_down_background = False
             
+        # enable selection
+        if self.timer is None:
+            self.timer = gobject.timeout_add(self.idle_threshold, self.on_timer_motion)
+        return False
+        
     def on_drag_data_received(self, unused_widget, unused_drag_context,
                                 x, y, selection_data, info, unused_timestamp):
         # The special value -1 means that this callback must
         # use pick_polygons to find out which face is to be updated
         #FIXME Redefining name ... ; anderen Namen verwenden
-        fs = selection.pick_polygons(self.widget, confstore.lighting, x, y)
-        if not fs:  # If nothing was actually pointed to,  then exit
-            return
-        swatch = fs.face
+        fs = self.pick_polygons(x, y)
+        # if fs is None the background was pointed to
+        swatch = fs and fs.face
         
         if info == RDRAG_COLOUR:
             #FIXME lenght and format available in PyGTK 2.14 and above.
@@ -416,34 +467,37 @@ class CubeArea (gobject.GObject):
                 debug("Received invalid color data")
                 return
                 
-            vals = [ord(selection_data.data[0]) | ord(selection_data.data[1]) << 8,
-                    ord(selection_data.data[2]) | ord(selection_data.data[3]) << 8,
-                    ord(selection_data.data[4]) | ord(selection_data.data[5]) << 8]
-            colour = gdk.Color()
-            colour.red = vals[0]
-            colour.green = vals[1]
-            colour.blue = vals[2]
+            color = gdk.Color()
+            color.red = ord(selection_data.data[0]) | ord(selection_data.data[1]) << 8
+            color.green = ord(selection_data.data[2]) | ord(selection_data.data[3]) << 8
+            color.blue = ord(selection_data.data[4]) | ord(selection_data.data[5]) << 8
             
-            drwBlock.face_rendering_set(swatch,
-                                            red=vals[0] / 65535.0,
-                                            green=vals[1] / 65535.0,
-                                            blue=vals[2] / 65535.0,
+            if swatch is not None:
+                drwBlock.face_rendering_set(swatch,
+                                            red=color.red_float,
+                                            green=color.green_float,
+                                            blue=color.blue_float,
                                             facetype=COLORED)
+                confstore.set_color(swatch, color)
+            else:
+                confstore.background_color = str(color)
                                             
-            confstore.set_color(swatch, colour)
             self.emit('texture-changed')
             self.render()
                 
         elif info == RDRAG_FILELIST:
-            start = selection_data.data.split("\r\n")
-            for s in start:
-                if s == "":
+            if swatch is None:
+                debug('Background image is not supported.')
+                return
+            uris = selection_data.data.split("\r\n")
+            for uri in uris:
+                if not uri:
                     continue
                     
-                fileobj = gio.file_parse_name(s)
-                filename = gio.file_parse_name(s).get_path()
-                if not fileobj.query_exists():
-                    debug('filename "%s" not found.' % filename)
+                fileobj = gio.File(uri=uri)
+                filename = fileobj.get_path()
+                if not filename or not fileobj.query_exists():
+                    debug('filename "%s" not found or not a local file.' % filename)
                     continue
                     
                 confstore.set_image_filename(swatch, filename)
@@ -457,30 +511,30 @@ class CubeArea (gobject.GObject):
         # Ignore all others
         
     def set_cursor(self):
-        if not self.setcursorPending:
+        if not self.setcursor_pending:
             gobject.idle_add(self.set_cursor_idle)
-            self.setcursorPending = 1
+            self.setcursor_pending = 1
             
     def set_cursor_idle(self):
-        fg = gdk.Color(65535, 65535, 65535) # White.
-        bg = gdk.Color(    0,     0,     0) # Black.
-        
         if self.is_item_selected() and not self.button_down_background:
-            data_bits, mask_bits, height, width, hot_x, hot_y = cursors.get_cursor(self.cursor_angle)
-            source = gdk.bitmap_create_from_data(None, data_bits, width, height)
-            mask = gdk.bitmap_create_from_data(None, mask_bits, width, height)
-            cursor = gdk.Cursor(source, mask, fg, bg, hot_x, hot_y)
+            if self.current_selection.quadrant is None:
+                index = -2
+            else:
+                angle = cube.get_cursor_angle(self.current_selection.block,
+                                              self.current_selection.face,
+                                              self.current_selection.quadrant)
+                index = int((angle+180) / 22.5 + 0.5) % 16
         else:
-            cursor = gdk.Cursor(gdk.CROSSHAIR)
-        self.widget.window.set_cursor(cursor)
+            index = -1
+        self.widget.window.set_cursor(self.cursors[index])
         
-        self.setcursorPending = 0
+        self.setcursor_pending = 0
         return False
         
     def render(self):
-        if not self.renderPending:
+        if not self.render_pending:
             gobject.idle_add(self.render_idle)
-            self.renderPending = 1
+            self.render_pending = 1
             
     def render_idle(self):
         # Reset the bit planes, and render the scene
@@ -494,62 +548,55 @@ class CubeArea (gobject.GObject):
         glarea_common.display()
         gdkgl.Drawable.swap_buffers(gldrawable)
         
-        self.renderPending = 0
+        self.render_pending = 0
         return False
         
     def set_lighting(self, enable):
         glarea_common.set_lighting(enable)
         self.render()
         
-    def rotate_cube(self, axis, dir_):
-        '''Rotate cube about axis (screen relative)'''
-        glarea_common.rotate_cube(axis, dir_)
+    def set_selection_mode(self, mode):
+        self.selection_mode = mode
+        
+    def set_background_color(self, color):
+        glarea_common.set_background_color(color.red_float, color.green_float, color.blue_float)
+        self.render()
+        
+    def reset_rotation(self):
+        '''Reset cube rotation'''
+        self.rotation_x, self.rotation_y = glarea_common.set_rotation_xy(
+                                            self.default_rotation_x, self.default_rotation_y)
         self.render()
         
     ### Animation
     
-    def animate_rotation(self, application, move_data):
-        self.animation_in_progress = 1
+    def animate_rotation(self, move_data, blocks, stop_after):
+        self.stop_requested = stop_after
         drwBlock.start_animate(move_data.axis, move_data.slice, move_data.dir)
-        blocks = list(application.current_movement.current_cube_state.identify_rotation_blocks(
-                                move_data.axis, move_data.slice))
-        assert blocks
+        if DEBUG_LEVEL:
+            assert blocks
         cube.set_animation_blocks(blocks)
-        application.playbarstate.playing(application.current_movement.mark_before)
-        gobject.timeout_add(picture_rate, self._on_animate, application)
+        self.animation_in_progress = 1
+        gobject.timeout_add(picture_rate, self._on_animate)
         
-    def _on_animate(self, application):
-        # how many degrees motion per frame
-        increment = 90.0 / (self.frameQty + 1)
-        
-        # decrement the current angle
-        unfinished = drwBlock.step_animate(increment)
-        
-        # and redraw it
+    def _on_animate(self):
         self.render()
         
-        if unfinished and not application.current_movement.abort_requested:
+        increment = 90.0 / (self.frameQty + 1)
+        unfinished = drwBlock.step_animate(increment)
+        if unfinished and not self.abort_requested:
+            return True
             # call this timeout again instead of return True; picture_rate may changed
-            gobject.timeout_add(picture_rate, self._on_animate, application)
+            #gobject.timeout_add(picture_rate, self._on_animate)
         else:
             # we have finished the animation sequence now
             drwBlock.end_animate()
-            self.apply_to_glmodel(application.current_movement.current_cube_state)
             self.animation_in_progress = 0
-            self.update_selection()
-            application.current_movement.abort_requested = 0
             
-            if not application.current_movement.stop_requested:
-                application.update_statusbar()
-                move_data = application.current_movement.request_play()
-                if move_data:
-                    # go again
-                    self.animate_rotation(application, move_data)
-                    return False
-                    
-            # no more animations
-            application.current_movement.stop_requested = 0
-            application.update_ui()
+            self.emit('end-animation', self.stop_requested)
+            
+            self.abort_requested = False
+            self.stop_requested = False
         return False
         
     def on_confstore_changed(self, key, *subkeys):
@@ -586,6 +633,8 @@ class CubeArea (gobject.GObject):
             self.set_lighting(confstore.lighting)
         elif key == 'dimension':
             pass
+        elif key == 'selection_mode':
+            self.set_selection_mode(confstore.selection_mode)
         elif key == 'colors':
             i = int(subkeys[0])
             if subkeys[1] == 'color':
@@ -608,6 +657,8 @@ class CubeArea (gobject.GObject):
             else:
                 debug('Unknown conf value changed:', subkeys[1])
             self.render()
+        elif key == 'background_color':
+            self.set_background_color(gdk.Color(confstore.background_color))
         else:
             debug('Unknown conf value changed:', key)
         
